@@ -375,7 +375,8 @@ on_greeter_begin_verification (GdmGreeterServer *greeter_server,
                                GdmFactorySlave  *slave)
 {
         g_debug ("GdmFactorySlave: begin verification");
-        gdm_session_setup (GDM_SESSION (slave->priv->session));
+        gdm_session_setup (GDM_SESSION (slave->priv->session),
+                           "gdm");
 }
 
 static void
@@ -385,6 +386,7 @@ on_greeter_begin_verification_for_user (GdmGreeterServer *greeter_server,
 {
         g_debug ("GdmFactorySlave: begin verification for user");
         gdm_session_setup_for_user (GDM_SESSION (slave->priv->session),
+                                    "gdm",
                                     username);
 }
 
@@ -479,7 +481,7 @@ run_greeter (GdmFactorySlave *slave)
         /* FIXME: send a signal back to the master */
 
         /* Run the init script. gdmslave suspends until script has terminated */
-        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", "gdm");
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init/Default", "gdm");
 
         slave->priv->greeter_server = gdm_greeter_server_new (display_id);
         g_signal_connect (slave->priv->greeter_server,
@@ -670,14 +672,13 @@ gdm_factory_slave_run (GdmFactorySlave *slave)
 static gboolean
 gdm_factory_slave_start (GdmSlave *slave)
 {
-        gboolean res;
         gboolean ret;
 
         ret = FALSE;
 
         g_debug ("GdmFactorySlave: Starting factory slave");
 
-        res = GDM_SLAVE_CLASS (gdm_factory_slave_parent_class)->start (slave);
+        GDM_SLAVE_CLASS (gdm_factory_slave_parent_class)->start (slave);
 
         GDM_FACTORY_SLAVE (slave)->priv->session = gdm_session_relay_new ();
         g_signal_connect (GDM_FACTORY_SLAVE (slave)->priv->session,
@@ -766,11 +767,9 @@ gdm_factory_slave_start (GdmSlave *slave)
 static gboolean
 gdm_factory_slave_stop (GdmSlave *slave)
 {
-        gboolean res;
-
         g_debug ("GdmFactorySlave: Stopping factory_slave");
 
-        res = GDM_SLAVE_CLASS (gdm_factory_slave_parent_class)->stop (slave);
+        GDM_SLAVE_CLASS (gdm_factory_slave_parent_class)->stop (slave);
 
         if (GDM_FACTORY_SLAVE (slave)->priv->session != NULL) {
                 gdm_session_relay_stop (GDM_FACTORY_SLAVE (slave)->priv->session);
@@ -809,10 +808,6 @@ gdm_factory_slave_set_property (GObject      *object,
                                const GValue *value,
                                GParamSpec   *pspec)
 {
-        GdmFactorySlave *self;
-
-        self = GDM_FACTORY_SLAVE (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -826,10 +821,6 @@ gdm_factory_slave_get_property (GObject    *object,
                                 GValue     *value,
                                 GParamSpec *pspec)
 {
-        GdmFactorySlave *self;
-
-        self = GDM_FACTORY_SLAVE (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -843,9 +834,6 @@ gdm_factory_slave_constructor (GType                  type,
                                GObjectConstructParam *construct_properties)
 {
         GdmFactorySlave      *factory_slave;
-        GdmFactorySlaveClass *klass;
-
-        klass = GDM_FACTORY_SLAVE_CLASS (g_type_class_peek (GDM_TYPE_FACTORY_SLAVE));
 
         factory_slave = GDM_FACTORY_SLAVE (G_OBJECT_CLASS (gdm_factory_slave_parent_class)->constructor (type,
                                                                                                          n_construct_properties,
