@@ -69,6 +69,7 @@ enum {
 };
 
 enum {
+        BEGIN_AUTO_LOGIN,
         BEGIN_VERIFICATION,
         BEGIN_VERIFICATION_FOR_USER,
         QUERY_ANSWER,
@@ -79,6 +80,8 @@ enum {
         CANCELLED,
         CONNECTED,
         DISCONNECTED,
+        START_SESSION_WHEN_READY,
+        START_SESSION_LATER,
         LAST_SIGNAL
 };
 
@@ -116,6 +119,40 @@ send_dbus_message (DBusConnection *connection,
         return sent;
 }
 
+static void
+send_dbus_string_and_int_signal (GdmGreeterServer *greeter_server,
+                                 const char       *name,
+                                 const char       *text,
+                                 int               number)
+{
+        DBusMessage    *message;
+        DBusMessageIter iter;
+        const char     *str;
+
+        if (text != NULL) {
+                str = text;
+        } else {
+                str = "";
+        }
+
+        g_return_if_fail (greeter_server != NULL);
+
+        message = dbus_message_new_signal (GDM_GREETER_SERVER_DBUS_PATH,
+                                           GDM_GREETER_SERVER_DBUS_INTERFACE,
+                                           name);
+
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &str);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &number);
+
+        g_debug ("GreeterServer: Sending %s (%s %d)", name, str, number);
+        if (! send_dbus_message (greeter_server->priv->greeter_connection, message)) {
+                g_debug ("GreeterServer: Could not send %s signal", name);
+        }
+
+        dbus_message_unref (message);
+}
+
 static void
 send_dbus_string_signal (GdmGreeterServer *greeter_server,
                          const char       *name,
@@ -221,6 +258,34 @@ gdm_greeter_server_selected_user_changed (GdmGreeterServer *greeter_server,
         send_dbus_string_signal (greeter_server, "SelectedUserChanged", username);
 }
 
+void
+gdm_greeter_server_default_language_name_changed (GdmGreeterServer *greeter_server,
+                                                  const char       *language_name)
+{
+        send_dbus_string_signal (greeter_server, "DefaultLanguageNameChanged", language_name);
+}
+
+void
+gdm_greeter_server_default_session_name_changed (GdmGreeterServer *greeter_server,
+                                                 const char       *session_name)
+{
+        send_dbus_string_signal (greeter_server, "DefaultSessionNameChanged", session_name);
+}
+
+void
+gdm_greeter_server_request_timed_login (GdmGreeterServer *greeter_server,
+                                        const char       *username,
+                                        int               delay)
+{
+        send_dbus_string_and_int_signal (greeter_server, "TimedLoginRequested", username, delay);
+}
+
+void
+gdm_greeter_server_user_authorized (GdmGreeterServer *greeter_server)
+{
+        send_dbus_void_signal (greeter_server, "UserAuthorized");
+}
+
 /* Note: Use abstract sockets like dbus does by default on Linux. Abstract
  * sockets are only available on Linux.
  */
@@ -267,6 +332,33 @@ handle_begin_verification (GdmGreeterServer *greeter_server,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+handle_begin_auto_login (GdmGreeterServer *greeter_server,
+                         DBusConnection   *connection,
+                         DBusMessage      *message)
+{
+        DBusMessage *reply;
+        DBusError    error;
+        const char  *text;
+
+
+        dbus_error_init (&error);
+        if (! dbus_message_get_args (message, &error,
+                                     DBUS_TYPE_STRING, &text,
+                                     DBUS_TYPE_INVALID)) {
+                g_warning ("ERROR: %s", error.message);
+        }
+
+        g_debug ("GreeterServer: BeginAutoLogin for '%s'", text);
+
+        reply = dbus_message_new_method_return (message);
+        dbus_connection_send (connection, reply, NULL);
+        dbus_message_unref (reply);
+
+        g_signal_emit (greeter_server, signals [BEGIN_AUTO_LOGIN], 0, text);
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
 
 static DBusHandlerResult
 handle_begin_verification_for_user (GdmGreeterServer *greeter_server,
@@ -479,6 +571,38 @@ handle_get_display_id (GdmGreeterServer *greeter_server,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+handle_start_session_when_ready (GdmGreeterServer *greeter_server,
+                                 DBusConnection   *connection,
+                                 DBusMessage      *message)
+{
+        DBusMessage *reply;
+        DBusError    error;
+        gboolean     should_start_session;
+
+        dbus_error_init (&error);
+        if (! dbus_message_get_args (message, &error,
+                                     DBUS_TYPE_BOOLEAN, &should_start_session,
+                                     DBUS_TYPE_INVALID)) {
+                g_warning ("ERROR: %s", error.message);
+        }
+
+        g_debug ("GreeterServer: %sStartSessionWhenReady",
+                 should_start_session? "" : "Don't ");
+
+        reply = dbus_message_new_method_return (message);
+        dbus_connection_send (connection, reply, NULL);
+        dbus_message_unref (reply);
+
+        if (should_start_session) {
+                g_signal_emit (greeter_server, signals [START_SESSION_WHEN_READY], 0);
+        } else {
+                g_signal_emit (greeter_server, signals [START_SESSION_LATER] ,0);
+        }
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 greeter_handle_child_message (DBusConnection *connection,
                               DBusMessage    *message,
@@ -490,6 +614,8 @@ greeter_handle_child_message (DBusConnection *connection,
                 return handle_begin_verification (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "BeginVerificationForUser")) {
                 return handle_begin_verification_for_user (greeter_server, connection, message);
+        } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "BeginAutoLogin")) {
+                return handle_begin_auto_login (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "AnswerQuery")) {
                 return handle_answer_query (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "SelectSession")) {
@@ -506,6 +632,8 @@ greeter_handle_child_message (DBusConnection *connection,
                 return handle_disconnect (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "GetDisplayId")) {
                 return handle_get_display_id (greeter_server, connection, message);
+        } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "StartSessionWhenReady")) {
+                return handle_start_session_when_ready (greeter_server, connection, message);
         }
 
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
@@ -535,6 +663,7 @@ do_introspect (DBusConnection *connection,
         xml = g_string_append (xml,
                                "  <interface name=\"org.gnome.DisplayManager.GreeterServer\">\n"
                                "    <method name=\"BeginVerification\">\n"
+                               "    <method name=\"BeginTimedLogin\">\n"
                                "    </method>\n"
                                "    <method name=\"BeginVerificationForUser\">\n"
                                "      <arg name=\"username\" direction=\"in\" type=\"s\"/>\n"
@@ -561,6 +690,9 @@ do_introspect (DBusConnection *connection,
                                "    <method name=\"GetDisplayId\">\n"
                                "      <arg name=\"id\" direction=\"out\" type=\"o\"/>\n"
                                "    </method>\n"
+                               "    <method name=\"StartSessionWhenReady\">\n"
+                               "      <arg name=\"should_start_session\" type=\"b\"/>\n"
+                               "    </method>\n"
                                "    <signal name=\"Info\">\n"
                                "      <arg name=\"text\" type=\"s\"/>\n"
                                "    </signal>\n"
@@ -576,10 +708,22 @@ do_introspect (DBusConnection *connection,
                                "    <signal name=\"SelectedUserChanged\">\n"
                                "      <arg name=\"username\" type=\"s\"/>\n"
                                "    </signal>\n"
+                               "    <signal name=\"DefaultLanguageNameChanged\">\n"
+                               "      <arg name=\"language_name\" type=\"s\"/>\n"
+                               "    </signal>\n"
+                               "    <signal name=\"DefaultSessionNameChanged\">\n"
+                               "      <arg name=\"session_name\" type=\"s\"/>\n"
+                               "    </signal>\n"
+                               "    <signal name=\"TimedLoginRequested\">\n"
+                               "      <arg name=\"username\" type=\"s\"/>\n"
+                               "      <arg name=\"delay\" type=\"i\"/>\n"
+                               "    </signal>\n"
                                "    <signal name=\"Ready\">\n"
                                "    </signal>\n"
                                "    <signal name=\"Reset\">\n"
                                "    </signal>\n"
+                               "    <signal name=\"UserAuthorized\">\n"
+                               "    </signal>\n"
                                "  </interface>\n");
 
         reply = dbus_message_new_method_return (message);
@@ -896,9 +1040,6 @@ gdm_greeter_server_constructor (GType                  type,
                                GObjectConstructParam *construct_properties)
 {
         GdmGreeterServer      *greeter_server;
-        GdmGreeterServerClass *klass;
-
-        klass = GDM_GREETER_SERVER_CLASS (g_type_class_peek (GDM_TYPE_GREETER_SERVER));
 
         greeter_server = GDM_GREETER_SERVER (G_OBJECT_CLASS (gdm_greeter_server_parent_class)->constructor (type,
                                                                                        n_construct_properties,
@@ -950,6 +1091,17 @@ gdm_greeter_server_class_init (GdmGreeterServerClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
+        signals [BEGIN_AUTO_LOGIN] =
+                g_signal_new ("begin-auto-login",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, begin_auto_login),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+                              G_TYPE_STRING);
         signals [BEGIN_VERIFICATION_FOR_USER] =
                 g_signal_new ("begin-verification-for-user",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -1046,6 +1198,28 @@ gdm_greeter_server_class_init (GdmGreeterServerClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
+
+        signals [START_SESSION_WHEN_READY] =
+                g_signal_new ("start-session-when-ready",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, start_session_when_ready),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
+        signals [START_SESSION_LATER] =
+                g_signal_new ("start-session-later",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, start_session_later),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
 }
 
 static void
