@@ -45,7 +45,7 @@ typedef struct
 {
         GObjectClass   parent_class;
 
-        void (* begin_timed_login)          (GdmGreeterServer  *greeter_server);
+        void (* begin_auto_login)           (GdmGreeterServer  *greeter_server);
         void (* begin_verification)         (GdmGreeterServer  *greeter_server);
         void (* begin_verification_for_user)(GdmGreeterServer  *greeter_server,
                                              const char        *username);
@@ -62,6 +62,8 @@ typedef struct
         void (* cancelled)                  (GdmGreeterServer  *greeter_server);
         void (* connected)                  (GdmGreeterServer  *greeter_server);
         void (* disconnected)               (GdmGreeterServer  *greeter_server);
+        void (* start_session_when_ready)   (GdmGreeterServer  *greeter_server);
+        void (* start_session_later)        (GdmGreeterServer  *greeter_server);
 } GdmGreeterServerClass;
 
 GType               gdm_greeter_server_get_type              (void);
@@ -89,6 +91,12 @@ void                gdm_greeter_server_default_language_name_changed (GdmGreeter
 void                gdm_greeter_server_default_session_name_changed (GdmGreeterServer *greeter_server,
                                                                      const char       *text);
 
+void                gdm_greeter_server_request_timed_login   (GdmGreeterServer *greeter_server,
+                                                              const char       *username,
+                                                              int               delay);
+void                gdm_greeter_server_user_authorized       (GdmGreeterServer *greeter_server);
+
+
 G_END_DECLS
 
 #endif /* __GDM_GREETER_SERVER_H */
