@@ -180,7 +180,8 @@ typedef struct {
 } RemoveDisplayData;
 
 static gboolean
-remove_display_for_connection (GdmDisplay        *display,
+remove_display_for_connection (char              *id,
+                               GdmDisplay        *display,
                                RemoveDisplayData *data)
 {
         g_assert (display != NULL);
@@ -207,8 +208,6 @@ remove_displays_for_connection (GdmManager *manager,
         data.service_name = service_name;
         data.manager = manager;
 
-        g_debug ("GdmManager: Removing display for service name: %s", service_name);
-
         gdm_display_store_foreach_remove (manager->priv->display_store,
                                           (GdmDisplayStoreFunc)remove_display_for_connection,
                                           &data);
@@ -224,9 +223,6 @@ bus_name_owner_changed (DBusGProxy  *bus_proxy,
         if (strlen (new_service_name) == 0) {
                 remove_displays_for_connection (manager, old_service_name);
         }
-
-        g_debug ("GdmManager: NameOwnerChanged: service_name='%s', old_service_name='%s' new_service_name='%s'",
-                   service_name, old_service_name, new_service_name);
 }
 
 static gboolean
@@ -320,9 +316,6 @@ gdm_manager_constructor (GType                  type,
                          GObjectConstructParam *construct_properties)
 {
         GdmManager      *manager;
-        GdmManagerClass *klass;
-
-        klass = GDM_MANAGER_CLASS (g_type_class_peek (GDM_TYPE_MANAGER));
 
         manager = GDM_MANAGER (G_OBJECT_CLASS (gdm_manager_parent_class)->constructor (type,
                                                                                        n_construct_properties,
@@ -373,7 +366,7 @@ gdm_manager_class_init (GdmManagerClass *klass)
                                          g_param_spec_boolean ("xdmcp-enabled",
                                                                NULL,
                                                                NULL,
-                                                               TRUE,
+                                                               FALSE,
                                                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
         g_type_class_add_private (klass, sizeof (GdmManagerPrivate));
