@@ -67,7 +67,7 @@
 #endif
 
 #ifndef GDM_SESSION_DEFAULT_PATH
-#define GDM_SESSION_DEFAULT_PATH "/usr/local/bin:/usr/bin:/bin:/usr/X11R6/bin"
+#define GDM_SESSION_DEFAULT_PATH "/usr/local/bin:/usr/bin:/bin"
 #endif
 
 #ifndef GDM_SESSION_ROOT_UID
@@ -88,6 +88,9 @@ enum {
         GDM_SESSION_WORKER_STATE_ACCREDITED,
         GDM_SESSION_WORKER_STATE_SESSION_OPENED,
         GDM_SESSION_WORKER_STATE_SESSION_STARTED,
+        GDM_SESSION_WORKER_STATE_REAUTHENTICATED,
+        GDM_SESSION_WORKER_STATE_REAUTHORIZED,
+        GDM_SESSION_WORKER_STATE_REACCREDITED,
 };
 
 struct GdmSessionWorkerPrivate
@@ -108,6 +111,7 @@ struct GdmSessionWorkerPrivate
         char             *display_device;
         char             *hostname;
         char             *username;
+        uid_t             uid;
         gboolean          password_is_required;
 
         int               cred_flags;
@@ -381,7 +385,9 @@ send_dbus_string_method (DBusConnection *connection,
                          error.message);
                 return FALSE;
         }
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (connection);
 
         return TRUE;
@@ -418,7 +424,9 @@ send_dbus_int_method (DBusConnection *connection,
                                                            -1,
                                                            &error);
         dbus_message_unref (message);
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (connection);
 
         if (dbus_error_is_set (&error)) {
@@ -456,7 +464,9 @@ send_dbus_void_method (DBusConnection *connection,
                                                            -1,
                                                            &error);
         dbus_message_unref (message);
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (connection);
 
         if (dbus_error_is_set (&error)) {
@@ -543,8 +553,9 @@ gdm_session_worker_update_username (GdmSessionWorker *worker)
         res = gdm_session_worker_get_username (worker, &username);
         if (res) {
                 g_debug ("GdmSessionWorker: old-username='%s' new-username='%s'",
-                         username != NULL ? username : "<unset>",
-                         worker->priv->username != NULL ? worker->priv->username : "<unset>");
+                         worker->priv->username != NULL ? worker->priv->username : "<unset>",
+                         username != NULL ? username : "<unset>");
+
 
                 gdm_session_auditor_set_username (worker->priv->auditor, worker->priv->username);
 
@@ -813,8 +824,10 @@ gdm_session_worker_pam_new_messages_handler (int                        number_o
                                                                        messages[i],
                                                                        &response_text);
                 if (!got_response) {
-                        memset (response_text, '\0', strlen (response_text));
-                        g_free (response_text);
+                        if (response_text != NULL) {
+                                memset (response_text, '\0', strlen (response_text));
+                                g_free (response_text);
+                        }
                         goto out;
                 }
 
@@ -1124,14 +1137,17 @@ gdm_session_worker_set_environment_variable (GdmSessionWorker *worker,
 }
 
 static void
-gdm_session_worker_update_environment_from_passwd_entry (GdmSessionWorker *worker,
-                                                         struct passwd    *passwd_entry)
+gdm_session_worker_update_environment_from_passwd_info (GdmSessionWorker *worker,
+                                                        uid_t             uid,
+                                                        gid_t             gid,
+                                                        const char       *home,
+                                                        const char       *shell)
 {
         gdm_session_worker_set_environment_variable (worker, "LOGNAME", worker->priv->username);
         gdm_session_worker_set_environment_variable (worker, "USER", worker->priv->username);
         gdm_session_worker_set_environment_variable (worker, "USERNAME", worker->priv->username);
-        gdm_session_worker_set_environment_variable (worker, "HOME", passwd_entry->pw_dir);
-        gdm_session_worker_set_environment_variable (worker, "SHELL", passwd_entry->pw_shell);
+        gdm_session_worker_set_environment_variable (worker, "HOME", home);
+        gdm_session_worker_set_environment_variable (worker, "SHELL", shell);
 }
 
 static gboolean
@@ -1142,28 +1158,56 @@ gdm_session_worker_environment_variable_is_set (GdmSessionWorker *worker,
 }
 
 static gboolean
-gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
-                                  GError           **error)
+_change_user (GdmSessionWorker  *worker,
+              uid_t              uid,
+              gid_t              gid)
 {
-        int            error_code;
+        gboolean ret;
+
+        ret = FALSE;
+
+#ifdef THE_MAN_PAGE_ISNT_LYING
+        /* pam_setcred wants to be called as the authenticated user
+         * but pam_open_session needs to be called as super-user.
+         *
+         * Set the real uid and gid to the user and give the user a
+         * temporary super-user effective id.
+         */
+        if (setreuid (uid, GDM_SESSION_ROOT_UID) < 0) {
+                return FALSE;
+        }
+#endif
+        worker->priv->uid = uid;
+
+        if (setgid (gid) < 0) {
+                return FALSE;
+        }
+
+        if (initgroups (worker->priv->username, gid) < 0) {
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
+static gboolean
+_lookup_passwd_info (const char *username,
+                     uid_t      *uidp,
+                     gid_t      *gidp,
+                     char      **homep,
+                     char      **shellp)
+{
+        gboolean       ret;
         struct passwd *passwd_entry;
         struct passwd  passwd_buffer;
         char          *aux_buffer;
         long           required_aux_buffer_size;
         gsize          aux_buffer_size;
 
+        ret = FALSE;
         aux_buffer = NULL;
         aux_buffer_size = 0;
 
-        if (worker->priv->username == NULL) {
-                error_code = PAM_USER_UNKNOWN;
-                g_set_error (error,
-                             GDM_SESSION_WORKER_ERROR,
-                             GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
-                             _("no user account available"));
-                goto out;
-        }
-
         required_aux_buffer_size = sysconf (_SC_GETPW_R_SIZE_MAX);
 
         if (required_aux_buffer_size < 0) {
@@ -1181,13 +1225,13 @@ gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
          */
         passwd_entry = NULL;
 #ifdef HAVE_POSIX_GETPWNAM_R
-        errno = getpwnam_r (worker->priv->username,
+        errno = getpwnam_r (username,
                             &passwd_buffer,
                             aux_buffer,
                             (size_t) aux_buffer_size,
                             &passwd_entry);
 #else
-        passwd_entry = getpwnam_r (worker->priv->username,
+        passwd_entry = getpwnam_r (username,
                                    &passwd_buffer,
                                    aux_buffer,
                                    (size_t) aux_buffer_size);
@@ -1195,59 +1239,102 @@ gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
 #endif /* !HAVE_POSIX_GETPWNAM_R */
 
         if (errno != 0) {
-                error_code = PAM_SYSTEM_ERR;
-                g_set_error (error,
-                             GDM_SESSION_WORKER_ERROR,
-                             GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
-                             "%s",
-                             g_strerror (errno));
+                g_warning ("%s", g_strerror (errno));
                 goto out;
         }
 
         if (passwd_entry == NULL) {
+                goto out;
+        }
+
+        if (uidp != NULL) {
+                *uidp = passwd_entry->pw_uid;
+        }
+        if (gidp != NULL) {
+                *gidp = passwd_entry->pw_gid;
+        }
+        if (homep != NULL) {
+                *homep = g_strdup (passwd_entry->pw_dir);
+        }
+        if (shellp != NULL) {
+                *shellp = g_strdup (passwd_entry->pw_shell);
+        }
+        ret = TRUE;
+ out:
+        if (aux_buffer != NULL) {
+                g_assert (aux_buffer_size > 0);
+                g_slice_free1 (aux_buffer_size, aux_buffer);
+        }
+
+        return ret;
+}
+
+static gboolean
+gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
+                                  GError           **error)
+{
+        gboolean ret;
+        gboolean res;
+        uid_t    uid;
+        gid_t    gid;
+        char    *shell;
+        char    *home;
+        int      error_code;
+
+        ret = FALSE;
+
+        if (worker->priv->username == NULL) {
+                g_debug ("GdmSessionWorker: Username not set");
                 error_code = PAM_USER_UNKNOWN;
                 g_set_error (error,
                              GDM_SESSION_WORKER_ERROR,
                              GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
-                             _("user account not available on system"));
+                             _("no user account available"));
                 goto out;
         }
 
-        gdm_session_worker_update_environment_from_passwd_entry (worker, passwd_entry);
+        home = NULL;
+        shell = NULL;
+        uid = 0;
+        gid = 0;
+        res = _lookup_passwd_info (worker->priv->username,
+                                   &uid,
+                                   &gid,
+                                   &home,
+                                   &shell);
+        if (! res) {
+                g_debug ("GdmSessionWorker: Unable to lookup account info");
+                g_set_error (error,
+                             GDM_SESSION_WORKER_ERROR,
+                             GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
+                             _("no user account available"));
+                goto out;
+        }
+
+        gdm_session_worker_update_environment_from_passwd_info (worker,
+                                                                uid,
+                                                                gid,
+                                                                home,
+                                                                shell);
 
         /* Let's give the user a default PATH if he doesn't already have one
          */
         if (!gdm_session_worker_environment_variable_is_set (worker, "PATH")) {
-                gdm_session_worker_set_environment_variable (worker, "PATH", GDM_SESSION_DEFAULT_PATH);
+                if (strcmp (BINDIR, "/usr/bin") == 0) {
+                        gdm_session_worker_set_environment_variable (worker, "PATH",
+                                                                     GDM_SESSION_DEFAULT_PATH);
+                } else {
+                        gdm_session_worker_set_environment_variable (worker, "PATH",
+                                                                     BINDIR ":" GDM_SESSION_DEFAULT_PATH);
+                }
         }
 
-        /* pam_setcred wants to be called as the authenticated user
-         * but pam_open_session needs to be called as super-user.
-         *
-         * Set the real uid and gid to the user and give the user a
-         * temporary super-user effective id.
-         */
-        if (setreuid (passwd_entry->pw_uid, GDM_SESSION_ROOT_UID) < 0) {
+        if (! _change_user (worker, uid, gid)) {
+                g_debug ("GdmSessionWorker: Unable to change to user");
                 error_code = PAM_SYSTEM_ERR;
                 g_set_error (error, GDM_SESSION_WORKER_ERROR,
                              GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
-                             "%s", g_strerror (errno));
-                goto out;
-        }
-
-        if (setgid (passwd_entry->pw_gid) < 0) {
-                error_code = PAM_SYSTEM_ERR;
-                g_set_error (error, GDM_SESSION_WORKER_ERROR,
-                             GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
-                             "%s", g_strerror (errno));
-                goto out;
-        }
-
-        if (initgroups (passwd_entry->pw_name, passwd_entry->pw_gid) < 0) {
-                error_code = PAM_SYSTEM_ERR;
-                g_set_error (error, GDM_SESSION_WORKER_ERROR,
-                             GDM_SESSION_WORKER_ERROR_GIVING_CREDENTIALS,
-                             "%s", g_strerror (errno));
+                             "%s", _("Unable to change to user"));
                 goto out;
         }
 
@@ -1262,23 +1349,19 @@ gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
                 goto out;
         }
 
-        gdm_session_auditor_report_user_accredited (worker->priv->auditor);
-
-        g_debug ("GdmSessionWorker: state ACCREDITED");
-        worker->priv->state = GDM_SESSION_WORKER_STATE_ACCREDITED;
+        ret = TRUE;
 
  out:
-        if (aux_buffer != NULL) {
-                g_assert (aux_buffer_size > 0);
-                g_slice_free1 (aux_buffer_size, aux_buffer);
-        }
-
-        if (error_code != PAM_SUCCESS) {
+        if (ret) {
+                g_debug ("GdmSessionWorker: state ACCREDITED");
+                ret = TRUE;
+                gdm_session_auditor_report_user_accredited (worker->priv->auditor);
+                worker->priv->state = GDM_SESSION_WORKER_STATE_ACCREDITED;
+        } else {
                 gdm_session_worker_uninitialize_pam (worker, error_code);
-                return FALSE;
         }
 
-        return TRUE;
+        return ret;
 }
 
 static void
@@ -1493,7 +1576,7 @@ gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
                 char  *home_dir;
                 int    fd;
 
-                if (setuid (getuid ()) < 0) {
+                if (setuid (worker->priv->uid) < 0) {
                         g_debug ("GdmSessionWorker: could not reset uid - %s", g_strerror (errno));
                         _exit (1);
                 }
@@ -1958,7 +2041,7 @@ state_change_idle (GdmSessionWorker *worker)
         int new_state;
 
         new_state = worker->priv->state + 1;
-        g_debug ("GdmSessionWorker: changing state to %s",
+        g_debug ("GdmSessionWorker: attempting to change state to %s",
                  get_state_name (new_state));
 
         worker->priv->state_change_idle_id = 0;
@@ -2021,6 +2104,7 @@ on_start_program (GdmSessionWorker *worker,
 
                 if (worker->priv->arguments != NULL) {
                         g_strfreev (worker->priv->arguments);
+                        worker->priv->arguments = NULL;
                 }
 
                 parse_error = NULL;
@@ -2145,16 +2229,36 @@ on_establish_credentials (GdmSessionWorker *worker,
 }
 
 static void
-on_renew_credentials (GdmSessionWorker *worker,
-                      DBusMessage      *message)
+on_reauthenticate (GdmSessionWorker *worker,
+                   DBusMessage      *message)
 {
-        /* FIXME: return error if not in AUTHORIZED state */
-
-        worker->priv->cred_flags = PAM_REINITIALIZE_CRED;
-
+        /* FIXME: return error if not in SESSION_STARTED state */
         queue_state_change (worker);
 }
 
+static void
+on_reauthorize (GdmSessionWorker *worker,
+                DBusMessage      *message)
+{
+        /* FIXME: return error if not in REAUTHENTICATED state */
+        queue_state_change (worker);
+}
+
+static void
+on_refresh_credentials (GdmSessionWorker *worker,
+                        DBusMessage      *message)
+{
+        int error_code;
+
+        /* FIXME: return error if not in REAUTHORIZED state */
+        g_debug ("GdmSessionWorker: refreshing credentials");
+
+        error_code = pam_setcred (worker->priv->pam_handle, PAM_REFRESH_CRED);
+        if (error_code != PAM_SUCCESS) {
+                g_debug ("GdmSessionWorker: %s", pam_strerror (worker->priv->pam_handle, error_code));
+        }
+}
+
 static DBusHandlerResult
 worker_dbus_handle_message (DBusConnection *connection,
                             DBusMessage    *message,
@@ -2184,10 +2288,14 @@ worker_dbus_handle_message (DBusConnection *connection,
                 on_authorize (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "EstablishCredentials")) {
                 on_establish_credentials (worker, message);
-        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "RenewCredentials")) {
-                on_renew_credentials (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "StartProgram")) {
                 on_start_program (worker, message);
+        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "Reauthenticate")) {
+                on_reauthenticate (worker, message);
+        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "Reauthorize")) {
+                on_reauthorize (worker, message);
+        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "RefreshCredentials")) {
+                on_refresh_credentials (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetEnvironmentVariable")) {
                 on_set_environment_variable (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetLanguageName")) {
