@@ -88,10 +88,6 @@ struct GdmSlavePrivate
         char            *parent_display_name;
         char            *parent_display_x11_authority_file;
 
-        /* user selected */
-        char            *selected_session;
-        char            *selected_language;
-
         DBusGProxy      *display_proxy;
         DBusGConnection *connection;
 };
@@ -337,6 +333,11 @@ gdm_slave_run_script (GdmSlave   *slave,
         g_ptr_array_foreach (env, (GFunc)g_free, NULL);
         g_ptr_array_free (env, TRUE);
 
+        if (! res) {
+                g_warning ("GdmSlave: Unable to run script: %s", error->message);
+                g_error_free (error);
+        }
+
         gdm_slave_whack_temp_auth_file (slave);
 
         if (WIFEXITED (status)) {
@@ -682,6 +683,8 @@ gdm_slave_add_user_authorization (GdmSlave   *slave,
                 *filenamep = NULL;
         }
 
+        g_debug ("GdmSlave: Requesting user authorization");
+
         error = NULL;
         res = dbus_g_proxy_call (slave->priv->display_proxy,
                                  "AddUserAuthorization",
@@ -690,10 +693,6 @@ gdm_slave_add_user_authorization (GdmSlave   *slave,
                                  G_TYPE_INVALID,
                                  G_TYPE_STRING, &filename,
                                  G_TYPE_INVALID);
-        if (filenamep != NULL) {
-                *filenamep = g_strdup (filename);
-        }
-        g_free (filename);
 
         if (! res) {
                 if (error != NULL) {
@@ -702,8 +701,15 @@ gdm_slave_add_user_authorization (GdmSlave   *slave,
                 } else {
                         g_warning ("Failed to add user authorization");
                 }
+        } else {
+                g_debug ("GdmSlave: Got user authorization: %s", filename);
         }
 
+        if (filenamep != NULL) {
+                *filenamep = g_strdup (filename);
+        }
+        g_free (filename);
+
         return res;
 }
 
@@ -831,12 +837,11 @@ x11_session_is_on_seat (GdmSlave        *slave,
         return ret;
 }
 
-static char *
-_get_primary_user_session_id (GdmSlave   *slave,
-                              const char *username)
+char *
+gdm_slave_get_primary_session_id_for_user (GdmSlave   *slave,
+                                           const char *username)
 {
         gboolean    res;
-        gboolean    ret;
         gboolean    can_activate_sessions;
         GError     *error;
         DBusGProxy *manager_proxy;
@@ -851,7 +856,6 @@ _get_primary_user_session_id (GdmSlave   *slave,
                 return NULL;
         }
 
-        ret = FALSE;
         manager_proxy = NULL;
         primary_ssid = NULL;
         sessions = NULL;
@@ -946,6 +950,7 @@ activate_session_id (GdmSlave   *slave,
         gboolean     ret;
 
         ret = FALSE;
+        reply = NULL;
 
         dbus_error_init (&local_error);
         message = dbus_message_new_method_call ("org.freedesktop.ConsoleKit",
@@ -978,6 +983,13 @@ activate_session_id (GdmSlave   *slave,
 
         ret = TRUE;
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return ret;
 }
 
@@ -1004,7 +1016,9 @@ session_unlock (GdmSlave   *slave,
                                                            message,
                                                            -1, &error);
         dbus_message_unref (message);
-        dbus_message_unref (reply);
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
         dbus_connection_flush (dbus_g_connection_get_connection (slave->priv->connection));
 
         if (dbus_error_is_set (&error)) {
@@ -1025,7 +1039,7 @@ gdm_slave_switch_to_user_session (GdmSlave   *slave,
 
         ret = FALSE;
 
-        ssid_to_activate = _get_primary_user_session_id (slave, username);
+        ssid_to_activate = gdm_slave_get_primary_session_id_for_user (slave, username);
         if (ssid_to_activate == NULL) {
                 g_debug ("GdmSlave: unable to determine session to activate");
                 goto out;
@@ -1209,12 +1223,9 @@ gdm_slave_constructor (GType                  type,
                        GObjectConstructParam *construct_properties)
 {
         GdmSlave      *slave;
-        GdmSlaveClass *klass;
         gboolean       res;
         const char    *id;
 
-        klass = GDM_SLAVE_CLASS (g_type_class_peek (GDM_TYPE_SLAVE));
-
         slave = GDM_SLAVE (G_OBJECT_CLASS (gdm_slave_parent_class)->constructor (type,
                                                                                  n_construct_properties,
                                                                                  construct_properties));
@@ -1225,6 +1236,8 @@ gdm_slave_constructor (GType                  type,
                 id = slave->priv->display_id + strlen ("/org/gnome/DisplayManager/Display");
         }
 
+        g_assert (id != NULL);
+
         slave->priv->id = g_strdup_printf ("/org/gnome/DisplayManager/Slave%s", id);
         g_debug ("GdmSlave: Registering %s", slave->priv->id);
 
@@ -1340,7 +1353,14 @@ gdm_slave_finalize (GObject *object)
 
         gdm_slave_real_stop (slave);
 
+        g_free (slave->priv->id);
         g_free (slave->priv->display_id);
+        g_free (slave->priv->display_name);
+        g_free (slave->priv->display_hostname);
+        g_free (slave->priv->display_seat_id);
+        g_free (slave->priv->display_x11_authority_file);
+        g_free (slave->priv->parent_display_name);
+        g_free (slave->priv->parent_display_x11_authority_file);
 
         G_OBJECT_CLASS (gdm_slave_parent_class)->finalize (object);
 }
