@@ -48,10 +48,6 @@ typedef struct
 
         void (* activated)      (GdmChooserWidget *widget);
         void (* deactivated)    (GdmChooserWidget *widget);
-
-#ifdef BUILD_ALLOCATION_HACK
-        gulong size_negotiation_handler;
-#endif
 } GdmChooserWidgetClass;
 
 typedef enum {
@@ -127,6 +123,14 @@ void                   gdm_chooser_widget_set_separator_position  (GdmChooserWid
                                                                    GdmChooserWidgetPosition  position);
 void                   gdm_chooser_widget_set_hide_inactive_items (GdmChooserWidget         *widget,
                                                                    gboolean                  should_hide);
+
+void                   gdm_chooser_widget_activate_selected_item (GdmChooserWidget *widget);
+
+int                    gdm_chooser_widget_get_number_of_items    (GdmChooserWidget *widget);
+void                   gdm_chooser_widget_activate_on_one_item   (GdmChooserWidget *widget,
+                                                                  gboolean          should_activate);
+void                   gdm_chooser_widget_propagate_pending_key_events (GdmChooserWidget *widget);
+
 G_END_DECLS
 
 #endif /* __GDM_CHOOSER_WIDGET_H */
