@@ -31,14 +31,20 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
+#include <gconf/gconf-client.h>
+
+#include "gdm-languages.h"
 #include "gdm-greeter-panel.h"
 #include "gdm-clock-widget.h"
 #include "gdm-language-option-widget.h"
 #include "gdm-session-option-widget.h"
 #include "gdm-a11y-preferences-dialog.h"
+#include "gdm-profile.h"
 
 #include "na-tray.h"
 
+#define KEY_DISABLE_A11Y_BUTTON "/apps/gdm/simple-greeter/disable-accessibility-button"
+
 #define GDM_GREETER_PANEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_PANEL, GdmGreeterPanelPrivate))
 
 struct GdmGreeterPanelPrivate
@@ -48,10 +54,15 @@ struct GdmGreeterPanelPrivate
         GtkWidget              *a11y_button;
         GtkWidget              *a11y_dialog;
         GtkWidget              *hbox;
+        GtkWidget              *alignment;
+        GtkWidget              *option_hbox;
         GtkWidget              *hostname_label;
         GtkWidget              *clock;
         GtkWidget              *language_option_widget;
         GtkWidget              *session_option_widget;
+
+        char                   *default_session_name;
+        char                   *default_language_name;
 };
 
 enum {
@@ -78,10 +89,6 @@ gdm_greeter_panel_set_property (GObject        *object,
                                 const GValue   *value,
                                 GParamSpec     *pspec)
 {
-        GdmGreeterPanel *self;
-
-        self = GDM_GREETER_PANEL (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -95,10 +102,6 @@ gdm_greeter_panel_get_property (GObject        *object,
                                 GValue         *value,
                                 GParamSpec     *pspec)
 {
-        GdmGreeterPanel *self;
-
-        self = GDM_GREETER_PANEL (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -112,26 +115,21 @@ gdm_greeter_panel_constructor (GType                  type,
                                GObjectConstructParam *construct_properties)
 {
         GdmGreeterPanel      *greeter_panel;
-        GdmGreeterPanelClass *klass;
 
-        klass = GDM_GREETER_PANEL_CLASS (g_type_class_peek (GDM_TYPE_GREETER_PANEL));
+        gdm_profile_start (NULL);
 
         greeter_panel = GDM_GREETER_PANEL (G_OBJECT_CLASS (gdm_greeter_panel_parent_class)->constructor (type,
                                                                                                          n_construct_properties,
                                                                                                          construct_properties));
 
+        gdm_profile_end (NULL);
+
         return G_OBJECT (greeter_panel);
 }
 
 static void
 gdm_greeter_panel_dispose (GObject *object)
 {
-        GdmGreeterPanel *greeter_panel;
-
-        greeter_panel = GDM_GREETER_PANEL (object);
-
-        g_debug ("Disposing greeter_panel");
-
         G_OBJECT_CLASS (gdm_greeter_panel_parent_class)->dispose (object);
 }
 
@@ -268,7 +266,9 @@ set_struts (GdmGreeterPanel *panel,
         /* bottom_end_x */
         data[11] = x + width;
 
+#if 0
         g_debug ("Setting strut: bottom=%lu bottom_start_x=%lu bottom_end_x=%lu", data[3], data[10], data[11]);
+#endif
 
         gdk_error_trap_push ();
 
@@ -320,11 +320,13 @@ update_geometry (GdmGreeterPanel *panel,
         panel->priv->geometry.x = geometry.x;
         panel->priv->geometry.y = geometry.y + geometry.height - panel->priv->geometry.height;
 
+#if 0
         g_debug ("Setting geometry x:%d y:%d w:%d h:%d",
                  panel->priv->geometry.x,
                  panel->priv->geometry.y,
                  panel->priv->geometry.width,
                  panel->priv->geometry.height);
+#endif
 
         update_struts (panel);
 }
@@ -473,11 +475,34 @@ on_a11y_button_clicked (GtkButton       *button,
         gtk_window_present (GTK_WINDOW (panel->priv->a11y_dialog));
 }
 
+static gboolean
+is_a11y_button_disabled (GdmGreeterPanel *panel)
+{
+        GConfClient *client;
+        GError      *error;
+        gboolean     disabled;
+
+        client = gconf_client_get_default ();
+        error = NULL;
+        disabled = gconf_client_get_bool (client, KEY_DISABLE_A11Y_BUTTON, &error);
+        if (error != NULL) {
+                g_debug ("GdmGreeterPanel: unable to get disable-accessibility-button configuration: %s", error->message);
+                g_error_free (error);
+        }
+        g_object_unref (client);
+
+        return disabled;
+}
+
 static void
 gdm_greeter_panel_init (GdmGreeterPanel *panel)
 {
         NaTray    *tray;
         GtkWidget *image;
+        GtkWidget *spacer;
+	AtkObject *atk_obj;
+
+        gdm_profile_start (NULL);
 
         panel->priv = GDM_GREETER_PANEL_GET_PRIVATE (panel);
 
@@ -493,35 +518,59 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
 
         gtk_window_set_keep_above (GTK_WINDOW (panel), TRUE);
         gtk_window_set_type_hint (GTK_WINDOW (panel), GDK_WINDOW_TYPE_HINT_DOCK);
-        gtk_window_set_opacity (GTK_WINDOW (panel), 0.75);
+        gtk_window_set_opacity (GTK_WINDOW (panel), 0.85);
 
         panel->priv->hbox = gtk_hbox_new (FALSE, 12);
-        gtk_container_set_border_width (GTK_CONTAINER (panel->priv->hbox), 2);
+        gtk_container_set_border_width (GTK_CONTAINER (panel->priv->hbox), 0);
         gtk_widget_show (panel->priv->hbox);
         gtk_container_add (GTK_CONTAINER (panel), panel->priv->hbox);
 
-        panel->priv->a11y_button = gtk_button_new ();
-        image = gtk_image_new_from_icon_name ("preferences-desktop-accessibility", GTK_ICON_SIZE_BUTTON);
-        gtk_container_add (GTK_CONTAINER (panel->priv->a11y_button), image);
-        gtk_widget_show (image);
-        gtk_widget_show (panel->priv->a11y_button);
-        g_signal_connect (G_OBJECT (panel->priv->a11y_button),
-                          "clicked",
-                          G_CALLBACK (on_a11y_button_clicked), panel);
+        if (! is_a11y_button_disabled (panel)) {
+                panel->priv->a11y_button = gtk_button_new ();
+                image = gtk_image_new_from_icon_name ("preferences-desktop-accessibility", GTK_ICON_SIZE_BUTTON);
+                gtk_container_add (GTK_CONTAINER (panel->priv->a11y_button), image);
+		atk_obj = gtk_widget_get_accessible (panel->priv->a11y_button);
+		if (GTK_IS_ACCESSIBLE (atk_obj))
+			atk_object_set_name (atk_obj, _("Accessibility Preferences"));
 
-        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->a11y_button, FALSE, FALSE, 6);
+                gtk_widget_show (image);
+                gtk_widget_show (panel->priv->a11y_button);
+                g_signal_connect (G_OBJECT (panel->priv->a11y_button),
+                                  "clicked",
+                                  G_CALLBACK (on_a11y_button_clicked), panel);
 
+                gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->a11y_button, FALSE, FALSE, 0);
+        }
+
+        panel->priv->alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->alignment, TRUE, TRUE, 0);
+        gtk_widget_show (panel->priv->alignment);
+
+        panel->priv->option_hbox = gtk_hbox_new (FALSE, 12);
+        gtk_widget_show (panel->priv->option_hbox);
+        gtk_container_add (GTK_CONTAINER (panel->priv->alignment), panel->priv->option_hbox);
+
+        spacer = gtk_label_new ("");
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), spacer, TRUE, TRUE, 6);
+        gtk_widget_show (spacer);
+
+        gdm_profile_start ("creating option widget");
         panel->priv->language_option_widget = gdm_language_option_widget_new ();
         g_signal_connect (G_OBJECT (panel->priv->language_option_widget),
                           "language-activated",
                           G_CALLBACK (on_language_activated), panel);
-        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->language_option_widget, FALSE, FALSE, 6);
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->language_option_widget, FALSE, FALSE, 6);
+        gdm_profile_end ("creating option widget");
 
         panel->priv->session_option_widget = gdm_session_option_widget_new ();
         g_signal_connect (G_OBJECT (panel->priv->session_option_widget),
                           "session-activated",
                           G_CALLBACK (on_session_activated), panel);
-        gtk_box_pack_start (GTK_BOX (panel->priv->hbox), panel->priv->session_option_widget, FALSE, FALSE, 6);
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->session_option_widget, FALSE, FALSE, 6);
+
+        spacer = gtk_label_new ("");
+        gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), spacer, TRUE, TRUE, 6);
+        gtk_widget_show (spacer);
 
         /* FIXME: we should only show hostname on panel when connected
            to a remote host */
@@ -533,7 +582,7 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
 
         panel->priv->clock = gdm_clock_widget_new ();
         gtk_box_pack_end (GTK_BOX (panel->priv->hbox),
-                          GTK_WIDGET (panel->priv->clock), FALSE, FALSE, 6);
+                            GTK_WIDGET (panel->priv->clock), FALSE, FALSE, 6);
         gtk_widget_show (panel->priv->clock);
 
         tray = na_tray_new_for_screen (gtk_window_get_screen (GTK_WINDOW (panel)),
@@ -542,6 +591,8 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
         gtk_widget_show (GTK_WIDGET (tray));
 
         gdm_greeter_panel_hide_user_options (panel);
+
+        gdm_profile_end (NULL);
 }
 
 static void
@@ -583,3 +634,55 @@ gdm_greeter_panel_hide_user_options (GdmGreeterPanel *panel)
         gtk_widget_hide (panel->priv->session_option_widget);
         gtk_widget_hide (panel->priv->language_option_widget);
 }
+
+void
+gdm_greeter_panel_reset (GdmGreeterPanel *panel)
+{
+        gdm_greeter_panel_set_default_language_name (panel, NULL);
+        gdm_greeter_panel_set_default_session_name (panel, NULL);
+        gdm_greeter_panel_hide_user_options (panel);
+}
+
+void
+gdm_greeter_panel_set_default_language_name (GdmGreeterPanel *panel,
+                                             const char      *language_name)
+{
+        char *normalized_language_name;
+
+        g_return_if_fail (GDM_IS_GREETER_PANEL (panel));
+
+        if (language_name != NULL) {
+                normalized_language_name = gdm_normalize_language_name (language_name);
+        } else {
+                normalized_language_name = NULL;
+        }
+
+        if (normalized_language_name != NULL &&
+            !gdm_option_widget_lookup_item (GDM_OPTION_WIDGET (panel->priv->language_option_widget),
+                                            normalized_language_name, NULL, NULL, NULL)) {
+                gdm_recent_option_widget_add_item (GDM_RECENT_OPTION_WIDGET (panel->priv->language_option_widget),
+                                                   normalized_language_name);
+        }
+
+        gdm_option_widget_set_default_item (GDM_OPTION_WIDGET (panel->priv->language_option_widget),
+                                            normalized_language_name);
+
+        g_free (normalized_language_name);
+}
+
+void
+gdm_greeter_panel_set_default_session_name (GdmGreeterPanel *panel,
+                                            const char      *session_name)
+{
+        g_return_if_fail (GDM_IS_GREETER_PANEL (panel));
+
+        if (session_name != NULL &&
+            !gdm_option_widget_lookup_item (GDM_OPTION_WIDGET (panel->priv->session_option_widget),
+                                            session_name, NULL, NULL, NULL)) {
+                g_warning ("Default session is not available");
+                return;
+        }
+
+        gdm_option_widget_set_default_item (GDM_OPTION_WIDGET (panel->priv->session_option_widget),
+                                            session_name);
+}
