@@ -25,6 +25,9 @@
 #include <unistd.h>
 #include <string.h>
 
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
@@ -37,14 +40,22 @@
 
 #include "gdm-session-manager.h"
 #include "gdm-session-client.h"
+#include "gdm-profile.h"
 
 #define GDM_GREETER_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_SESSION, GdmGreeterSessionPrivate))
 
-#define KEY_GDM_A11Y_DIR            "/apps/gdm/simple-greeter/accessibility"
+#define GSD_DBUS_NAME      "org.gnome.SettingsDaemon"
+#define GSD_DBUS_PATH      "/org/gnome/SettingsDaemon"
+#define GSD_DBUS_INTERFACE "org.gnome.SettingsDaemon"
+
+#define KEY_GDM_DIR                 "/apps/gdm/simple-greeter"
+#define KEY_GDM_A11Y_DIR             KEY_GDM_DIR "/accessibility"
 #define KEY_SCREEN_KEYBOARD_ENABLED  KEY_GDM_A11Y_DIR "/screen_keyboard_enabled"
 #define KEY_SCREEN_MAGNIFIER_ENABLED KEY_GDM_A11Y_DIR "/screen_magnifier_enabled"
 #define KEY_SCREEN_READER_ENABLED    KEY_GDM_A11Y_DIR "/screen_reader_enabled"
 
+#define KEY_WM_USE_COMPIZ            KEY_GDM_DIR "/wm_use_compiz"
+
 struct GdmGreeterSessionPrivate
 {
         GdmGreeterClient      *client;
@@ -137,6 +148,24 @@ on_default_session_name_changed (GdmGreeterClient  *client,
                                                     text);
 }
 
+static void
+on_timed_login_requested (GdmGreeterClient  *client,
+                          const char        *text,
+                          int                delay,
+                          GdmGreeterSession *session)
+{
+        g_debug ("GdmGreeterSession: timed login requested for user %s (in %d seconds)", text, delay);
+        gdm_greeter_login_window_request_timed_login (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), text, delay);
+}
+
+static void
+on_user_authorized (GdmGreeterClient  *client,
+                    GdmGreeterSession *session)
+{
+        g_debug ("GdmGreeterSession: user authorized");
+        gdm_greeter_login_window_user_authorized (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
+}
+
 static void
 on_info_query (GdmGreeterClient  *client,
                const char        *text,
@@ -158,10 +187,12 @@ on_secret_info_query (GdmGreeterClient  *client,
 }
 
 static void
-on_begin_timed_login (GdmGreeterLoginWindow *login_window,
-                    GdmGreeterSession     *session)
+on_begin_auto_login (GdmGreeterLoginWindow *login_window,
+                     const char            *username,
+                     GdmGreeterSession     *session)
 {
-        gdm_greeter_client_call_begin_timed_login (session->priv->client);
+        gdm_greeter_client_call_begin_auto_login (session->priv->client,
+                                                  username);
 }
 
 static void
@@ -230,11 +261,20 @@ on_disconnected (GdmGreeterLoginWindow *login_window,
         gdm_greeter_client_call_disconnect (session->priv->client);
 }
 
+static void
+on_start_session (GdmGreeterLoginWindow *login_window,
+                  GdmGreeterSession     *session)
+{
+        gdm_greeter_client_call_start_session_when_ready (session->priv->client, TRUE);
+}
+
 static void
 toggle_panel (GdmSessionManager *manager,
               gboolean           enabled,
               GdmGreeterSession *session)
 {
+        gdm_profile_start (NULL);
+
         if (enabled) {
                 session->priv->panel = gdm_greeter_panel_new ();
 
@@ -253,6 +293,8 @@ toggle_panel (GdmSessionManager *manager,
                 gtk_widget_destroy (session->priv->panel);
                 session->priv->panel = NULL;
         }
+
+        gdm_profile_end (NULL);
 }
 
 static void
@@ -260,6 +302,8 @@ toggle_login_window (GdmSessionManager *manager,
                      gboolean           enabled,
                      GdmGreeterSession *session)
 {
+        gdm_profile_start (NULL);
+
         if (enabled) {
                 gboolean is_local;
 
@@ -268,8 +312,8 @@ toggle_login_window (GdmSessionManager *manager,
                 session->priv->login_window = gdm_greeter_login_window_new (is_local);
 
                 g_signal_connect (session->priv->login_window,
-                                  "begin-timed-login",
-                                  G_CALLBACK (on_begin_timed_login),
+                                  "begin-auto-login",
+                                  G_CALLBACK (on_begin_auto_login),
                                   session);
                 g_signal_connect (session->priv->login_window,
                                   "begin-verification",
@@ -295,11 +339,16 @@ toggle_login_window (GdmSessionManager *manager,
                                   "disconnected",
                                   G_CALLBACK (on_disconnected),
                                   session);
+                g_signal_connect (session->priv->login_window,
+                                  "start-session",
+                                  G_CALLBACK (on_start_session),
+                                  session);
                 gtk_widget_show (session->priv->login_window);
         } else {
                 gtk_widget_destroy (session->priv->login_window);
                 session->priv->login_window = NULL;
         }
+        gdm_profile_end (NULL);
 }
 
 static gboolean
@@ -308,12 +357,15 @@ launch_compiz (GdmGreeterSession *session)
         GError  *error;
         gboolean ret;
 
+        gdm_profile_start (NULL);
         g_debug ("GdmGreeterSession: Launching compiz");
 
         ret = FALSE;
 
+        g_setenv ("LIBGL_ALWAYS_INDIRECT", "1", TRUE);
+
         error = NULL;
-        g_spawn_command_line_async ("gtk-window-decorator --replace", &error);
+        g_spawn_command_line_async ("gtk-window-decorator", &error);
         if (error != NULL) {
                 g_warning ("Error starting WM: %s", error->message);
                 g_error_free (error);
@@ -321,7 +373,7 @@ launch_compiz (GdmGreeterSession *session)
         }
 
         error = NULL;
-        g_spawn_command_line_async ("compiz --replace", &error);
+        g_spawn_command_line_async ("compiz --replace glib gconf", &error);
         if (error != NULL) {
                 g_warning ("Error starting WM: %s", error->message);
                 g_error_free (error);
@@ -333,6 +385,7 @@ launch_compiz (GdmGreeterSession *session)
         /* FIXME: should try to detect if it actually works */
 
  out:
+        gdm_profile_end (NULL);
         return ret;
 }
 
@@ -342,6 +395,8 @@ launch_metacity (GdmGreeterSession *session)
         GError  *error;
         gboolean ret;
 
+        gdm_profile_start (NULL);
+
         g_debug ("GdmGreeterSession: Launching metacity");
 
         ret = FALSE;
@@ -357,15 +412,26 @@ launch_metacity (GdmGreeterSession *session)
         ret = TRUE;
 
  out:
+        gdm_profile_end (NULL);
         return ret;
 }
 
 static void
 start_window_manager (GdmGreeterSession *session)
 {
-        if (! launch_metacity (session)) {
-                launch_compiz (session);
+        gboolean     use_compiz;
+        GConfClient *client;
+
+        client = gconf_client_get_default ();
+        use_compiz = gconf_client_get_bool (client, KEY_WM_USE_COMPIZ, NULL);
+        g_object_unref (client);
+        if (use_compiz) {
+                if (launch_compiz (session)) {
+                        return;
+                }
         }
+
+        launch_metacity (session);
 }
 
 static void
@@ -480,7 +546,7 @@ setup_at_tools (GdmGreeterSession *session)
         gdm_session_client_set_try_exec (session->priv->screen_reader_client,
                                          "orca");
         gdm_session_client_set_command (session->priv->screen_reader_client,
-                                        "orca -n");
+                                        "orca -n -d main-window -d magnifier");
         enabled = gconf_client_get_bool (client, KEY_SCREEN_READER_ENABLED, NULL);
         gdm_session_client_set_enabled (session->priv->screen_reader_client,
                                         enabled);
@@ -510,6 +576,98 @@ setup_at_tools (GdmGreeterSession *session)
         g_object_unref (client);
 }
 
+static gboolean
+send_dbus_string_method (DBusConnection *connection,
+                         const char     *method,
+                         const char     *payload)
+{
+        DBusError       error;
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusMessageIter iter;
+        const char     *str;
+
+        if (payload != NULL) {
+                str = payload;
+        } else {
+                str = "";
+        }
+
+        g_debug ("GdmGreeterSession: Calling %s", method);
+        message = dbus_message_new_method_call (GSD_DBUS_NAME,
+                                                GSD_DBUS_PATH,
+                                                GSD_DBUS_INTERFACE,
+                                                method);
+        if (message == NULL) {
+                g_warning ("Couldn't allocate the D-Bus message");
+                return FALSE;
+        }
+
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter,
+                                        DBUS_TYPE_STRING,
+                                        &str);
+
+        dbus_error_init (&error);
+        reply = dbus_connection_send_with_reply_and_block (connection,
+                                                           message,
+                                                           -1,
+                                                           &error);
+
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&error)) {
+                g_warning ("%s %s raised: %s\n",
+                           method,
+                           error.name,
+                           error.message);
+                return FALSE;
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+        dbus_connection_flush (connection);
+
+        return TRUE;
+}
+
+static gboolean
+activate_settings_daemon (GdmGreeterSession *session)
+{
+        gboolean         ret;
+        gboolean         res;
+        DBusError        local_error;
+        DBusConnection  *connection;
+
+        gdm_profile_start (NULL);
+
+        ret = FALSE;
+
+        g_debug ("GdmGreeterLoginWindow: activating settings daemon");
+
+        dbus_error_init (&local_error);
+        connection = dbus_bus_get (DBUS_BUS_SESSION, &local_error);
+        if (connection == NULL) {
+                g_debug ("Failed to connect to the D-Bus daemon: %s", local_error.message);
+                dbus_error_free (&local_error);
+                goto out;
+        }
+
+        res = send_dbus_string_method (connection,
+                                       "StartWithSettingsPrefix",
+                                       "/apps/gdm/simple-greeter/settings-manager-plugins");
+        if (! res) {
+                g_warning ("Couldn't start settings daemon");
+                goto out;
+        }
+        ret = TRUE;
+        g_debug ("GdmGreeterLoginWindow: settings daemon started");
+ out:
+        gdm_profile_end (NULL);
+
+        return ret;
+}
+
 static gboolean
 start_settings_daemon (GdmGreeterSession *session)
 {
@@ -540,7 +698,9 @@ toggle_all_levels (GdmSessionManager *manager,
                    GdmGreeterSession *session)
 {
         if (enabled) {
-                start_settings_daemon (session);
+                if (!activate_settings_daemon (session)) {
+                        start_settings_daemon (session);
+                }
                 start_window_manager (session);
         } else {
         }
@@ -554,10 +714,14 @@ gdm_greeter_session_start (GdmGreeterSession *session,
 
         g_return_val_if_fail (GDM_IS_GREETER_SESSION (session), FALSE);
 
+        gdm_profile_start (NULL);
+
         res = gdm_greeter_client_start (session->priv->client, error);
 
         gdm_session_manager_set_level (session->priv->manager, GDM_SESSION_LEVEL_LOGIN_WINDOW);
 
+        gdm_profile_end (NULL);
+
         return res;
 }
 
@@ -665,6 +829,7 @@ gdm_greeter_session_event_handler(GdkEvent          *event,
 static void
 gdm_greeter_session_init (GdmGreeterSession *session)
 {
+        gdm_profile_start (NULL);
 
         session->priv = GDM_GREETER_SESSION_GET_PRIVATE (session);
 
@@ -721,6 +886,14 @@ gdm_greeter_session_init (GdmGreeterSession *session)
                           "default-session-name-changed",
                           G_CALLBACK (on_default_session_name_changed),
                           session);
+        g_signal_connect (session->priv->client,
+                          "timed-login-requested",
+                          G_CALLBACK (on_timed_login_requested),
+                          session);
+        g_signal_connect (session->priv->client,
+                          "user-authorized",
+                          G_CALLBACK (on_user_authorized),
+                          session);
 
         /* We want to listen for panel mnemonics even if the
          * login window is focused, so we intercept them here.
@@ -730,6 +903,8 @@ gdm_greeter_session_init (GdmGreeterSession *session)
 
         /* FIXME: we should really do this in settings daemon */
         setup_at_tools (session);
+
+        gdm_profile_end (NULL);
 }
 
 static void
