@@ -29,6 +29,8 @@
 #include <locale.h>
 #include <sys/stat.h>
 
+#include <fontconfig/fontconfig.h>
+
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
@@ -42,7 +44,7 @@
 
 struct GdmLanguageChooserWidgetPrivate
 {
-        gpointer            dummy;
+        guint               languages_added : 1;
 };
 
 static void     gdm_language_chooser_widget_class_init  (GdmLanguageChooserWidgetClass *klass);
@@ -67,8 +69,9 @@ gdm_language_chooser_widget_get_current_language_name (GdmLanguageChooserWidget
         language_name = gdm_chooser_widget_get_active_item (GDM_CHOOSER_WIDGET (widget));
 
         if (language_name == NULL) {
-                language_name = g_strdup (g_get_language_names ()[0]);
+                language_name = g_strdup (setlocale (LC_MESSAGES, NULL));
         }
+
         return language_name;
 }
 
@@ -93,52 +96,82 @@ gdm_language_chooser_widget_set_current_language_name (GdmLanguageChooserWidget
         g_free (name);
 }
 
-static void
-gdm_language_chooser_widget_dispose (GObject *object)
-{
-        GdmLanguageChooserWidget *widget;
-
-        widget = GDM_LANGUAGE_CHOOSER_WIDGET (object);
-
-        G_OBJECT_CLASS (gdm_language_chooser_widget_parent_class)->dispose (object);
-}
-
-static void
-gdm_language_chooser_widget_class_init (GdmLanguageChooserWidgetClass *klass)
-{
-        GObjectClass   *object_class = G_OBJECT_CLASS (klass);
-
-        object_class->dispose = gdm_language_chooser_widget_dispose;
-        object_class->finalize = gdm_language_chooser_widget_finalize;
-
-        g_type_class_add_private (klass, sizeof (GdmLanguageChooserWidgetPrivate));
-}
-
 static void
 gdm_language_chooser_widget_add_language (GdmLanguageChooserWidget *widget,
                                           const char               *name)
 {
         char *language;
         char *normalized_name;
+        char *readable_language;
 
         normalized_name = gdm_normalize_language_name (name);
-        language = gdm_get_language_from_name (normalized_name);
+        language = gdm_get_language_from_name (normalized_name, normalized_name);
+        readable_language = gdm_get_language_from_name (normalized_name, NULL);
 
         if (language != NULL) {
                 gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
                                              normalized_name,
                                              NULL,
                                              language,
-                                             "",
+                                             readable_language,
                                              0,
                                              FALSE,
                                              FALSE);
                 g_free (language);
         }
+        g_free (readable_language);
 
         g_free (normalized_name);
 }
 
+static gboolean
+language_has_font (const char *locale)
+{
+  FcPattern   *pattern;
+  FcObjectSet *object_set;
+  FcFontSet   *font_set;
+  char        *language_code;
+  gboolean     is_displayable;
+
+  is_displayable = FALSE;
+  pattern = NULL;
+  object_set = NULL;
+  font_set = NULL;
+
+  gdm_parse_language_name (locale, &language_code, NULL, NULL, NULL);
+
+  pattern = FcPatternBuild (NULL, FC_LANG, FcTypeString, language_code, NULL);
+
+  if (pattern == NULL)
+    goto done;
+
+  object_set = FcObjectSetBuild (NULL, NULL);
+
+  if (object_set == NULL)
+    goto done;
+
+  font_set = FcFontList (NULL, pattern, object_set);
+
+  if (font_set == NULL)
+    goto done;
+
+  is_displayable = (font_set->nfont > 0);
+
+done:
+
+  if (font_set != NULL)
+    FcFontSetDestroy (font_set);
+
+  if (object_set != NULL)
+    FcObjectSetDestroy (object_set);
+
+  if (pattern != NULL)
+    FcPatternDestroy (pattern);
+
+  g_free (language_code);
+  return is_displayable;
+}
+
 static void
 add_available_languages (GdmLanguageChooserWidget *widget)
 {
@@ -148,6 +181,9 @@ add_available_languages (GdmLanguageChooserWidget *widget)
         language_names = gdm_get_all_language_names ();
 
         for (i = 0; language_names[i] != NULL; i++) {
+                if (!language_has_font (language_names[i])) {
+                        continue;
+                }
                 gdm_language_chooser_widget_add_language (widget,
                                                           language_names[i]);
         }
@@ -155,6 +191,40 @@ add_available_languages (GdmLanguageChooserWidget *widget)
         g_strfreev (language_names);
 }
 
+static void
+gdm_language_chooser_widget_dispose (GObject *object)
+{
+        G_OBJECT_CLASS (gdm_language_chooser_widget_parent_class)->dispose (object);
+}
+
+static void
+gdm_language_chooser_widget_realize (GtkWidget *widget)
+{
+        GdmLanguageChooserWidget *chooser;
+
+        chooser = GDM_LANGUAGE_CHOOSER_WIDGET (widget);
+
+        GTK_WIDGET_CLASS (gdm_language_chooser_widget_parent_class)->realize (widget);
+
+        if (!chooser->priv->languages_added) {
+                add_available_languages (chooser);
+                chooser->priv->languages_added = TRUE;
+        }
+}
+
+static void
+gdm_language_chooser_widget_class_init (GdmLanguageChooserWidgetClass *klass)
+{
+        GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+        object_class->dispose = gdm_language_chooser_widget_dispose;
+        object_class->finalize = gdm_language_chooser_widget_finalize;
+        widget_class->realize = gdm_language_chooser_widget_realize;
+
+        g_type_class_add_private (klass, sizeof (GdmLanguageChooserWidgetPrivate));
+}
+
 static void
 gdm_language_chooser_widget_init (GdmLanguageChooserWidget *widget)
 {
@@ -162,8 +232,6 @@ gdm_language_chooser_widget_init (GdmLanguageChooserWidget *widget)
 
         gdm_chooser_widget_set_separator_position (GDM_CHOOSER_WIDGET (widget),
                                                    GDM_CHOOSER_WIDGET_POSITION_TOP);
-
-        add_available_languages (widget);
 }
 
 static void
