@@ -72,6 +72,8 @@ struct _GdmUser {
         char           *shell;
         GList          *sessions;
         gulong          login_frequency;
+
+        GFileMonitor   *icon_monitor;
 };
 
 typedef struct _GdmUserClass
@@ -306,6 +308,61 @@ gdm_user_class_init (GdmUserClass *class)
                               G_TYPE_NONE, 0);
 }
 
+
+static void
+on_icon_monitor_changed (GFileMonitor     *monitor,
+                         GFile            *file,
+                         GFile            *other_file,
+                         GFileMonitorEvent event_type,
+                         GdmUser          *user)
+{
+        g_debug ("Icon changed: %d", event_type);
+
+        if (event_type != G_FILE_MONITOR_EVENT_CHANGED &&
+            event_type != G_FILE_MONITOR_EVENT_CREATED) {
+                return;
+        }
+
+        _gdm_user_icon_changed (user);
+}
+
+static void
+update_icon_monitor (GdmUser *user)
+{
+        GFile  *file;
+        GError *error;
+        char   *path;
+
+        if (user->home_dir == NULL) {
+                return;
+        }
+
+        if (user->icon_monitor != NULL) {
+                g_file_monitor_cancel (user->icon_monitor);
+                user->icon_monitor = NULL;
+        }
+
+        path = g_build_filename (user->home_dir, ".face", NULL);
+        g_debug ("adding monitor for '%s'", path);
+        file = g_file_new_for_path (path);
+        error = NULL;
+        user->icon_monitor = g_file_monitor_file (file,
+                                                  G_FILE_MONITOR_NONE,
+                                                  NULL,
+                                                  &error);
+        if (user->icon_monitor != NULL) {
+                g_signal_connect (user->icon_monitor,
+                                  "changed",
+                                  G_CALLBACK (on_icon_monitor_changed),
+                                  user);
+        } else {
+                g_warning ("Unable to monitor %s: %s", path, error->message);
+                g_error_free (error);
+        }
+        g_object_unref (file);
+        g_free (path);
+}
+
 static void
 gdm_user_init (GdmUser *user)
 {
@@ -322,6 +379,8 @@ gdm_user_finalize (GObject *object)
 
         user = GDM_USER (object);
 
+        g_file_monitor_cancel (user->icon_monitor);
+
         g_free (user->user_name);
         g_free (user->real_name);
 
@@ -419,6 +478,8 @@ _gdm_user_update (GdmUser             *user,
                 g_object_notify (G_OBJECT (user), "shell");
         }
 
+        update_icon_monitor (user);
+
         g_object_thaw_notify (G_OBJECT (user));
 }
 
@@ -545,42 +606,53 @@ gdm_user_get_login_frequency (GdmUser *user)
         return user->login_frequency;
 }
 
-gint
+int
 gdm_user_collate (GdmUser *user1,
                   GdmUser *user2)
 {
-        const gchar *str1, *str2;
+        const char *str1;
+        const char *str2;
+        gulong      num1;
+        gulong      num2;
 
         g_return_val_if_fail (user1 == NULL || GDM_IS_USER (user1), 0);
         g_return_val_if_fail (user2 == NULL || GDM_IS_USER (user2), 0);
 
-        if (!user1 && user2)
-                return -1;
-
-        if (user1 && !user2)
-                return 1;
-
-        if (!user1 && !user2)
-                return 0;
-
-        if (user1->real_name)
+        if (user1->real_name != NULL) {
                 str1 = user1->real_name;
-        else
+        } else {
                 str1 = user1->user_name;
+        }
 
-        if (user2->real_name)
+        if (user2->real_name != NULL) {
                 str2 = user2->real_name;
-        else
+        } else {
                 str2 = user2->user_name;
+        }
 
-        if (!str1 && str2)
+        num1 = user1->login_frequency;
+        num2 = user2->login_frequency;
+        g_debug ("Login freq 1=%u 2=%u", (guint)num1, (guint)num2);
+        if (num1 > num2) {
                 return -1;
+        }
 
-        if (str1 && !str2)
+        if (num1 < num2) {
                 return 1;
+        }
 
-        if (!str1 && !str2)
+        /* if login frequency is equal try names */
+        if (str1 == NULL && str2 != NULL) {
+                return -1;
+        }
+
+        if (str1 != NULL && str2 == NULL) {
+                return 1;
+        }
+
+        if (str1 == NULL && str2 == NULL) {
                 return 0;
+        }
 
         return g_utf8_collate (str1, str2);
 }
