@@ -52,15 +52,20 @@ typedef struct _GdmAppletData
 
         GConfClient    *client;
         GdmUserManager *manager;
+        GdmUser        *user;
 
         GtkWidget      *menubar;
-        GtkWidget      *imglabel;
+        GtkWidget      *menuitem;
         GtkWidget      *menu;
         GtkWidget      *separator_item;
         GtkWidget      *login_screen_item;
         GSList         *items;
 
+        gboolean        active_only;
+
         guint           client_notify_lockdown_id;
+
+        guint           user_icon_changed_id;
         guint           user_notify_id;
         GQuark          user_menu_item_quark;
         gint8           pixel_size;
@@ -104,33 +109,6 @@ about_me_cb (BonoboUIComponent *ui_container,
         }
 }
 
-static GladeXML *
-get_glade_xml (const char *root)
-{
-        GladeXML *xml;
-
-        xml = glade_xml_new (GLADEDIR "/gdm-user-switch-applet.glade", root, NULL);
-
-        if (xml == NULL) {
-                GtkWidget *dialog;
-
-                dialog = gtk_message_dialog_new (NULL,
-                                                 0,
-                                                 GTK_MESSAGE_ERROR,
-                                                 GTK_BUTTONS_CLOSE,
-                                                 _("Missing Required File"));
-                gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                                          _("The User Selector's interfaces file, `%s', could not be opened. It is likely that this application was not properly installed or configured."),
-                                                          GLADEDIR "/gdm-user-switch-applet.glade");
-                gtk_dialog_run (GTK_DIALOG (dialog));
-                gtk_widget_destroy (dialog);
-                bonobo_main_quit ();
-                return NULL;
-        }
-
-        return xml;
-}
-
 /*
  * gnome-panel/applets/wncklet/window-menu.c:window_filter_button_press()
  *
@@ -407,6 +385,21 @@ applet_key_press_event_cb (GtkWidget   *widget,
         return FALSE;
 }
 
+static void
+set_item_text_angle_and_alignment (GtkWidget *item,
+                                   double     text_angle,
+                                   float      xalign,
+                                   float      yalign)
+{
+        GtkWidget *label;
+
+        label = GTK_BIN (item)->child;
+
+        gtk_label_set_angle (GTK_LABEL (label), text_angle);
+
+        gtk_misc_set_alignment (GTK_MISC (label), xalign, yalign);
+}
+
 /*
  * gnome-panel/applets/wncklet/window-menu.c:window_menu_size_allocate()
  *
@@ -417,18 +410,23 @@ applet_key_press_event_cb (GtkWidget   *widget,
 static void
 applet_size_allocate_cb (GtkWidget     *widget,
                          GtkAllocation *allocation,
-                         gpointer       data)
+                         GdmAppletData *adata)
 {
-        GdmAppletData *adata;
-        GList *children;
-        GtkWidget *top_item;
+        GList            *children;
+        GtkWidget        *top_item;
         PanelAppletOrient orient;
-        gint item_border;
-        gint pixel_size;
-        gdouble text_angle = 0.0;
-        GtkPackDirection pack_direction = GTK_PACK_DIRECTION_LTR;
+        gint              item_border;
+        gint              pixel_size;
+        gdouble           text_angle;
+        GtkPackDirection  pack_direction;
+        float             text_xalign;
+        float             text_yalign;
+
+        pack_direction = GTK_PACK_DIRECTION_LTR;
+        text_angle = 0.0;
+        text_xalign = 0.0;
+        text_yalign = 0.5;
 
-        adata = data;
         children = gtk_container_get_children (GTK_CONTAINER (adata->menubar));
         top_item = GTK_WIDGET (children->data);
         g_list_free (children);
@@ -443,20 +441,22 @@ applet_size_allocate_cb (GtkWidget     *widget,
         case PANEL_APPLET_ORIENT_DOWN:
                 gtk_widget_set_size_request (top_item, -1, allocation->height);
                 pixel_size = allocation->height - item_border;
-                pack_direction = GTK_PACK_DIRECTION_RTL;
-                text_angle = 0.0;
                 break;
         case PANEL_APPLET_ORIENT_LEFT:
                 gtk_widget_set_size_request (top_item, allocation->width, -1);
                 pixel_size = allocation->width - item_border;
                 pack_direction = GTK_PACK_DIRECTION_TTB;
                 text_angle = 270.0;
+                text_xalign = 0.5;
+                text_yalign = 0.0;
                 break;
         case PANEL_APPLET_ORIENT_RIGHT:
                 gtk_widget_set_size_request (top_item, allocation->width, -1);
                 pixel_size = allocation->width - item_border;
-                pack_direction = GTK_PACK_DIRECTION_TTB;
+                pack_direction = GTK_PACK_DIRECTION_BTT;
                 text_angle = 90.0;
+                text_xalign = 0.5;
+                text_yalign = 0.0;
                 break;
         default:
                 g_assert_not_reached ();
@@ -465,13 +465,13 @@ applet_size_allocate_cb (GtkWidget     *widget,
 
         gtk_menu_bar_set_pack_direction (GTK_MENU_BAR (adata->menubar),
                                          pack_direction);
+        gtk_menu_bar_set_child_pack_direction (GTK_MENU_BAR (adata->menubar),
+                                               pack_direction);
 
-        if (GTK_IS_IMAGE (adata->imglabel)) {
-                gtk_image_set_pixel_size (GTK_IMAGE (adata->imglabel),
-                                          pixel_size - 4);
-        } else {
-                gtk_label_set_angle (GTK_LABEL (adata->imglabel), text_angle);
-        }
+        set_item_text_angle_and_alignment (adata->menuitem,
+                                           text_angle,
+                                           text_xalign,
+                                           text_yalign);
 }
 
 
@@ -480,6 +480,12 @@ gdm_applet_data_free (GdmAppletData *adata)
 {
         gconf_client_notify_remove (adata->client, adata->client_notify_lockdown_id);
 
+        g_signal_handler_disconnect (adata->user, adata->user_notify_id);
+        g_signal_handler_disconnect (adata->user, adata->user_icon_changed_id);
+
+        if (adata->user != NULL) {
+                g_object_unref (adata->user);
+        }
         g_object_unref (adata->client);
         g_object_unref (adata->manager);
         g_object_unref (tooltips);
@@ -580,8 +586,9 @@ sort_menu (GdmAppletData *adata)
         n_items = 0;
         items = adata->items;
         while (items) {
-                if (GTK_WIDGET_VISIBLE (items->data))
+                if (GTK_WIDGET_VISIBLE (items->data)) {
                         n_items++;
+                }
 
                 items = items->next;
         }
@@ -595,10 +602,14 @@ sort_menu (GdmAppletData *adata)
         column = 0;
         count = 0;
         items = adata->items;
-        while (items) {
+        while (items != NULL) {
                 if (GTK_WIDGET_VISIBLE (items->data)) {
-                        gtk_menu_attach (GTK_MENU (adata->menu), items->data,
-                                         column, column + 1, row, row + 1);
+                        gtk_menu_attach (GTK_MENU (adata->menu),
+                                         items->data,
+                                         column,
+                                         column + 1,
+                                         row,
+                                         row + 1);
                         row++;
                         if (row > n_rows) {
                                 row = 0;
@@ -610,7 +621,8 @@ sort_menu (GdmAppletData *adata)
                          * have to set that explicitly.
                          */
                         gtk_menu_reorder_child (GTK_MENU (adata->menu),
-                                                items->data, count++);
+                                                items->data,
+                                                count++);
                 }
 
                 items = items->next;
@@ -704,11 +716,14 @@ menuitem_style_set_cb (GtkWidget *menuitem,
 
 
 static void
-user_notify_display_name_cb (GObject    *object,
-                             GParamSpec *pspec,
-                             gpointer    data)
+user_notify_display_name_cb (GObject       *object,
+                             GParamSpec    *pspec,
+                             GdmAppletData *adata)
 {
-        gtk_label_set_text (data, gdm_user_get_real_name (GDM_USER (object)));
+        GtkWidget *label;
+
+        label = gtk_bin_get_child (GTK_BIN (adata->menuitem));
+        gtk_label_set_text (GTK_LABEL (label), gdm_user_get_real_name (GDM_USER (object)));
 }
 
 
@@ -844,21 +859,44 @@ user_item_activate_cb (GtkWidget     *menuitem,
 }
 
 static void
-user_sessions_changed_cb (GdmUser       *user,
-                          GdmAppletData *adata)
+update_user_item_visibility (GdmAppletData  *adata,
+                             GdmUser        *user)
 {
         GtkWidget *menuitem;
 
+        g_debug ("Updating menu item visibility for %s", gdm_user_get_user_name (user));
+
         menuitem = g_object_get_qdata (G_OBJECT (user), adata->user_menu_item_quark);
         if (menuitem == NULL) {
                 return;
         }
 
-        gtk_widget_show (menuitem);
+        if (adata->active_only) {
+                guint num_sessions;
+
+                num_sessions = gdm_user_get_num_sessions (user);
+                g_debug ("Sessions changed for %s num=%u", gdm_user_get_user_name (user), num_sessions);
+
+                if (num_sessions > 0) {
+                        gtk_widget_show (menuitem);
+                } else {
+                        gtk_widget_hide (menuitem);
+                }
+        } else {
+                gtk_widget_show (menuitem);
+        }
 
         sort_menu (adata);
 }
 
+static void
+on_user_sessions_changed (GdmUser       *user,
+                          GdmAppletData *adata)
+{
+        g_debug ("Sessions changed for %s", gdm_user_get_user_name (user));
+        update_user_item_visibility (adata, user);
+}
+
 static void
 add_user (GdmAppletData  *adata,
           GdmUser        *user)
@@ -886,16 +924,26 @@ add_user (GdmAppletData  *adata,
 
         g_signal_connect (user,
                           "sessions-changed",
-                          G_CALLBACK (user_sessions_changed_cb),
+                          G_CALLBACK (on_user_sessions_changed),
                           adata);
 }
 
 static void
-manager_user_added_cb (GdmUserManager *manager,
+on_manager_user_added (GdmUserManager *manager,
                        GdmUser        *user,
                        GdmAppletData  *adata)
 {
         add_user (adata, user);
+        update_user_item_visibility (adata, user);
+        sort_menu (adata);
+}
+
+static void
+on_manager_user_is_logged_in_changed (GdmUserManager *manager,
+                                      GdmUser        *user,
+                                      GdmAppletData  *adata)
+{
+        update_user_item_visibility (adata, user);
         sort_menu (adata);
 }
 
@@ -936,6 +984,67 @@ client_notify_lockdown_func (GConfClient   *client,
         }
 }
 
+static void
+reset_icon (GdmAppletData *adata)
+{
+        GdkPixbuf *pixbuf;
+        GtkWidget *image;
+
+        if (adata->user == NULL || !gtk_widget_has_screen (GTK_WIDGET (adata->menuitem))) {
+                return;
+        }
+
+        pixbuf = gdm_user_render_icon (adata->user, 24);
+        if (pixbuf != NULL) {
+                image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (adata->menuitem));
+                gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+                g_object_unref (pixbuf);
+        }
+}
+
+static void
+on_user_icon_changed (GdmUser         *user,
+                      GdmAppletData   *adata)
+{
+        g_debug ("User icon changed");
+        reset_icon (adata);
+}
+
+static void
+setup_current_user (GdmAppletData *adata)
+{
+        const char *name;
+
+        adata->user = gdm_user_manager_get_user_by_uid (adata->manager, getuid ());
+        if (adata->user != NULL) {
+                g_object_ref (adata->user);
+                name = gdm_user_get_real_name (adata->user);
+        } else {
+                name = _("Unknown");
+        }
+
+        adata->menuitem = gtk_image_menu_item_new_with_label (name);
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (adata->menuitem),
+                                       gtk_image_new ());
+        gtk_menu_shell_append (GTK_MENU_SHELL (adata->menubar), adata->menuitem);
+        gtk_widget_show (adata->menuitem);
+
+        if (adata->user != NULL) {
+                reset_icon (adata);
+
+                adata->user_icon_changed_id =
+                        g_signal_connect (adata->user,
+                                          "icon-changed",
+                                          G_CALLBACK (on_user_icon_changed),
+                                          adata);
+                adata->user_notify_id =
+                        g_signal_connect (adata->user,
+                                         "notify::display-name",
+                                         G_CALLBACK (user_notify_display_name_cb),
+                                         adata);
+        }
+}
+
 static gboolean
 fill_applet (PanelApplet *applet)
 {
@@ -947,8 +1056,6 @@ fill_applet (PanelApplet *applet)
                 BONOBO_UI_VERB_END
         };
         static gboolean    first_time = FALSE;
-        GtkWidget         *menuitem;
-        GtkWidget         *hbox;
         GSList            *users;
         char              *tmp;
         BonoboUIComponent *popup_component;
@@ -977,6 +1084,9 @@ fill_applet (PanelApplet *applet)
         adata = g_new0 (GdmAppletData, 1);
         adata->applet = applet;
 
+        /* Until we add user selecting to GDM */
+        adata->active_only = TRUE;
+
         adata->client = gconf_client_get_default ();
         adata->manager = gdm_user_manager_ref_default ();
 
@@ -1073,29 +1183,12 @@ fill_applet (PanelApplet *applet)
         gtk_container_add (GTK_CONTAINER (applet), adata->menubar);
         gtk_widget_show (adata->menubar);
 
-        menuitem = gtk_menu_item_new ();
-        gtk_menu_shell_append (GTK_MENU_SHELL (adata->menubar), menuitem);
-        gtk_widget_show (menuitem);
+        setup_current_user (adata);
 
-        hbox = gtk_hbox_new (FALSE, 0);
-        gtk_container_add (GTK_CONTAINER (menuitem), hbox);
-        gtk_widget_show (hbox);
 
-        {
-                GdmUser *user;
-
-                user = gdm_user_manager_get_user_by_uid (adata->manager, getuid ());
-                adata->imglabel = gtk_label_new (gdm_user_get_real_name (user));
-                adata->user_notify_id = g_signal_connect (user,
-                                                          "notify::display-name",
-                                                          G_CALLBACK (user_notify_display_name_cb),
-                                                          adata->imglabel);
-                gtk_box_pack_start (GTK_BOX (hbox), adata->imglabel, TRUE, TRUE, 0);
-                gtk_widget_show (adata->imglabel);
-        }
 
         adata->menu = gtk_menu_new ();
-        gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), adata->menu);
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (adata->menuitem), adata->menu);
         g_signal_connect (adata->menu, "style-set",
                           G_CALLBACK (menu_style_set_cb), adata);
         g_signal_connect (adata->menu, "show",
@@ -1113,13 +1206,18 @@ fill_applet (PanelApplet *applet)
         users = gdm_user_manager_list_users (adata->manager);
         while (users != NULL) {
                 add_user (adata, users->data);
+                update_user_item_visibility (adata, users->data);
 
                 users = g_slist_delete_link (users, users);
         }
 
         g_signal_connect (adata->manager,
                           "user-added",
-                          G_CALLBACK (manager_user_added_cb),
+                          G_CALLBACK (on_manager_user_added),
+                          adata);
+        g_signal_connect (adata->manager,
+                          "user-is-logged-in-changed",
+                          G_CALLBACK (on_manager_user_is_logged_in_changed),
                           adata);
 
         adata->separator_item = gtk_separator_menu_item_new ();
