@@ -52,6 +52,7 @@ static gboolean    debug_in         = FALSE;
 static gboolean    authenticate     = FALSE;
 static gboolean    startnew         = FALSE;
 static gboolean    monte_carlo_pi   = FALSE;
+static gboolean    show_version     = FALSE;
 static char      **args_remaining   = NULL;
 
 /* Keep all config options for compatibility even if they are noops */
@@ -63,10 +64,23 @@ GOptionEntry options [] = {
         { "authenticate", 'a', 0, G_OPTION_ARG_NONE, &authenticate, N_("Authenticate before running --command"), NULL },
         { "startnew", 's', 0, G_OPTION_ARG_NONE, &startnew, N_("Start new flexible session; do not show popup"), NULL },
         { "monte-carlo-pi", 0, 0, G_OPTION_ARG_NONE, &monte_carlo_pi, NULL, NULL },
+        { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, N_("Version of this application"), NULL },
         { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &args_remaining, NULL, NULL },
         { NULL }
 };
 
+#define GDM_FLEXISERVER_ERROR gdm_flexiserver_error_quark ()
+static GQuark
+gdm_flexiserver_error_quark (void)
+{
+        static GQuark ret = 0;
+        if (ret == 0) {
+                ret = g_quark_from_static_string ("gdm_flexiserver_error");
+        }
+
+        return ret;
+}
+
 static gboolean
 is_program_in_path (const char *program)
 {
@@ -87,10 +101,11 @@ maybe_lock_screen (void)
         char      *command;
         GdkScreen *screen;
 
-        if (is_program_in_path ("gnome-screensaver-command"))
+        if (is_program_in_path ("gnome-screensaver-command")) {
                 use_gscreensaver = TRUE;
-        else if (! is_program_in_path ("xscreensaver-command"))
+        } else if (! is_program_in_path ("xscreensaver-command")) {
                 return;
+        }
 
         if (use_gscreensaver) {
                 command = g_strdup ("gnome-screensaver-command --lock");
@@ -137,7 +152,8 @@ calc_pi (void)
 }
 
 static gboolean
-create_transient_display (DBusConnection *connection)
+create_transient_display (DBusConnection *connection,
+                          GError        **error)
 {
         DBusError       local_error;
         DBusMessage    *message;
@@ -147,6 +163,7 @@ create_transient_display (DBusConnection *connection)
         const char     *value;
 
         ret = FALSE;
+        reply = NULL;
 
         dbus_error_init (&local_error);
         message = dbus_message_new_method_call (GDM_DBUS_NAME,
@@ -154,6 +171,7 @@ create_transient_display (DBusConnection *connection)
                                                 GDM_DBUS_LOCAL_DISPLAY_FACTORY_INTERFACE,
                                                 "CreateTransientDisplay");
         if (message == NULL) {
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "Out of memory.");
                 goto out;
         }
 
@@ -165,6 +183,7 @@ create_transient_display (DBusConnection *connection)
         if (reply == NULL) {
                 if (dbus_error_is_set (&local_error)) {
                         g_warning ("Unable to create transient display: %s", local_error.message);
+                        g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "%s", local_error.message);
                         dbus_error_free (&local_error);
                         goto out;
                 }
@@ -174,10 +193,15 @@ create_transient_display (DBusConnection *connection)
         dbus_message_iter_get_basic (&iter, &value);
         g_debug ("Started %s", value);
 
-        dbus_message_unref (reply);
-
         ret = TRUE;
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return ret;
 }
 
@@ -193,6 +217,7 @@ get_current_session_id (DBusConnection *connection,
         const char     *value;
 
         ret = FALSE;
+        reply = NULL;
 
         dbus_error_init (&local_error);
         message = dbus_message_new_method_call (CK_NAME,
@@ -221,10 +246,16 @@ get_current_session_id (DBusConnection *connection,
         if (session_id != NULL) {
                 *session_id = g_strdup (value);
         }
-        dbus_message_unref (reply);
 
         ret = TRUE;
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return ret;
 }
 
@@ -241,6 +272,7 @@ get_seat_id_for_session (DBusConnection *connection,
         const char     *value;
 
         ret = FALSE;
+        reply = NULL;
 
         dbus_error_init (&local_error);
         message = dbus_message_new_method_call (CK_NAME,
@@ -269,10 +301,16 @@ get_seat_id_for_session (DBusConnection *connection,
         if (seat_id != NULL) {
                 *seat_id = g_strdup (value);
         }
-        dbus_message_unref (reply);
 
         ret = TRUE;
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return ret;
 }
 
@@ -306,6 +344,7 @@ activate_session_id (DBusConnection *connection,
         gboolean     ret;
 
         ret = FALSE;
+        reply = NULL;
 
         g_debug ("Switching to session %s", session_id);
 
@@ -340,6 +379,13 @@ activate_session_id (DBusConnection *connection,
 
         ret = TRUE;
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return ret;
 }
 
@@ -355,6 +401,7 @@ session_is_login_window (DBusConnection *connection,
         const char     *value;
 
         ret = FALSE;
+        reply = NULL;
 
         dbus_error_init (&local_error);
         message = dbus_message_new_method_call (CK_NAME,
@@ -385,10 +432,15 @@ session_is_login_window (DBusConnection *connection,
                 goto out;
         }
 
-        dbus_message_unref (reply);
-
         ret = TRUE;
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return ret;
 }
 
@@ -403,6 +455,7 @@ seat_can_activate_sessions (DBusConnection *connection,
         gboolean        can_activate;
 
         can_activate = FALSE;
+        reply = NULL;
 
         dbus_error_init (&local_error);
         message = dbus_message_new_method_call (CK_NAME,
@@ -428,9 +481,15 @@ seat_can_activate_sessions (DBusConnection *connection,
 
         dbus_message_iter_init (reply, &iter);
         dbus_message_iter_get_basic (&iter, &can_activate);
-        dbus_message_unref (reply);
 
  out:
+        if (message != NULL) {
+                dbus_message_unref (message);
+        }
+        if (reply != NULL) {
+                dbus_message_unref (reply);
+        }
+
         return can_activate;
 }
 
@@ -601,6 +660,7 @@ goto_login_session (GError **error)
         connection = dbus_bus_get (DBUS_BUS_SYSTEM, &local_error);
         if (connection == NULL) {
                 g_debug ("Failed to connect to the D-Bus daemon: %s", local_error.message);
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "%s", local_error.message);
                 dbus_error_free (&local_error);
                 return FALSE;
         }
@@ -611,6 +671,8 @@ goto_login_session (GError **error)
         seat_id = get_current_seat_id (connection);
         if (seat_id == NULL || seat_id[0] == '\0') {
                 g_debug ("seat id is not set; can't switch sessions");
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, _("Could not identify the current session."));
+
                 return FALSE;
         }
 
@@ -623,7 +685,7 @@ goto_login_session (GError **error)
         }
 
         if (! ret) {
-                res = create_transient_display (connection);
+                res = create_transient_display (connection, error);
                 if (res) {
                         ret = TRUE;
                 }
@@ -649,6 +711,12 @@ main (int argc, char *argv[])
         g_option_context_parse (ctx, &argc, &argv, NULL);
         g_option_context_free (ctx);
 
+
+        if (show_version) {
+                g_print ("%s %s\n", argv [0], VERSION);
+                exit (1);
+        }
+
         /* don't support commands */
         if (send_command != NULL) {
                 g_warning ("No longer supported");
@@ -687,7 +755,7 @@ main (int argc, char *argv[])
                 dialog = gtk_message_dialog_new (NULL,
                                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_MESSAGE_ERROR,
-                                                 GTK_BUTTONS_OK,
+                                                 GTK_BUTTONS_CLOSE,
                                                  "%s", _("Unable to start new display"));
 
                 gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
@@ -695,6 +763,7 @@ main (int argc, char *argv[])
                 g_free (message);
 
                 gtk_window_set_title (GTK_WINDOW (dialog), "");
+                gtk_window_set_icon_name (GTK_WINDOW (dialog), "session-properties");
                 gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
                 gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
 
